@@ -23,14 +23,16 @@
 #  All Rights Reserved.
 #  Contributor(s): ______________________________________.
 
-import pkg_resources
 import pyparsing as pp
 import sys
 import traceback
 from codeop import CommandCompiler
+import pwc
 import pwc.base as base
 import __builtin__
 import string
+import base
+import itertools
 
 __all__ = ["Interpreter", "Console", "interact"]
 
@@ -46,42 +48,53 @@ SET_TERM_CMD = 'SET TERM'
 #--- Classes
 
 class Interpreter(object):
-    """Interpreter based on code.InteractiveInterpreter from Python Library.
+    """Interpreter based on :class:`code.InteractiveInterpreter` from Python Library.
 
-    Attributes:
-    ui_provider     UserInterfaceProvider instance used by interpreter.
-    locals          the dictionary in which code will be executed.
-    compile         CommandCompiler instance.
-    linecont        Line continuation marker.
-    terminator      Default terminator string for terminated commands
-    commands        Disctionary of installed commands, key is command name.
-    full_grammar    Complete grammar for internal commands.
-    check_grammar   Grammar for internal commands used to check whether 
+    *Attributes:*
+    
+    :packages:      List of package interface objects.
+    :ui_provider:   UserInterfaceProvider instance used by interpreter.
+    :locals:        the dictionary in which code will be executed.
+    :compile:       CommandCompiler instance.
+    :linecont:      Line continuation marker.
+    :terminator:    Default terminator string for terminated commands
+    :commands:      Disctionary of installed commands, key is command name.
+    :full_grammar:  Complete grammar for internal commands.
+    :check_grammar: Grammar for internal commands used to check whether 
                     internal commands starts on given line.
     """
 
     terminator = ';'
     commands = {}
+    debug = False
     
-    def __init__(self,locals=None,ui_provider=None):
+    def __init__(self,packages=[],locals=None,ui_provider=None,ui_factory=None):
         """Constructor.
 
-        The optional 'locals' argument specifies the dictionary in
-        which code will be executed; it defaults to a newly created
-        dictionary with key "__name__" set to "__console__" and key
-        "__doc__" set to None.
+        :packages: Optional list of extension packages; defaults to empty list.
+        :locals: Optional dictionary in which the user code will be executed; 
+                 it defaults to a newly created dictionary with key "__name__"
+                 set to "__console__" and key "__doc__" set to None.
         
-        The optional ui_provider argument specifies the UserInterfaceProvider;
-        it defaults to base.UserInterfaceProvider.
+        :ui_provider: Optional UserInterfaceProvider; defaults to 
+                      base.UserInterfaceProvider.
+        :ui_factory: Optional User Interface Factory callable that creates UI
+                     elements with required interface.
         """
 
+        self.packages = packages
+        
         if ui_provider is None:
-            ui_provider = base.UserInterfaceProvider()
+            ui_provider = base.UserInterfaceProvider(packages,ui_factory)
         self.ui_provider = ui_provider
-        self.__errout = self.ui_provider.getLineDisplay(context="main.stderr")
-        self.__out = self.ui_provider.getLineDisplay(context="main")
+        
+        # Displays for use in Interpreter
+        self.__errout = self.ui_provider.getDisplay("main.stderr")
+        self.__out = self.ui_provider.getDisplay("main")
+        
         if locals is None:
-            locals = {"__name__": "__console__", "__doc__": None,"shell":self}
+            locals = {"__name__": "__console__", "__doc__": None,"shell":self,
+                      "ui": ui_provider}
         self.locals = locals
         self.compile = CommandCompiler()
         self.linecont = '\\'
@@ -89,12 +102,26 @@ class Interpreter(object):
         # Install our own displayhook that uses ui_provider
         self.__displayhook = sys.displayhook
         sys.displayhook = self._displayhook
-        self.__display = self.ui_provider.getObjectDisplay(context="main.displayhook")
+        self.__display = self.ui_provider.getDisplay("main.displayhook",base.UI_OBJECT)
+
+        # Call Controller extenders
+        for extender in itertools.chain(*(p.controller_extensions() for p 
+                                          in self.packages 
+                                          if hasattr(p,'controller_extensions'))):
+            try:
+                extender(self)
+            except:
+                ## ToDo: Handle exceptions in extender call
+                raise
 
         # Find and instantiate commands
-        for command in pkg_resources.iter_entry_points("powerconsole.commands"):
+        commands = [pwc.stdcmd.cmdList,pwc.stdcmd.cmdHelp,pwc.stdcmd.cmdShow,
+                    pwc.stdcmd.cmdRun]
+        commands.extend(itertools.chain(*(p.commands() for p 
+                                     in self.packages if hasattr(p,'commands'))))
+        for command in commands:
             try:
-                cmd = command.load()(self)
+                cmd = command(self)
                 name = cmd.getName()
             except:
                 ## ToDo: Handle exceptions in command initialization
@@ -147,15 +174,13 @@ class Interpreter(object):
     def _rebuildGrammar(self,grammar_type):
         if grammar_type == base.GRAMMAR_FULL:
             self.full_grammar = self.__buildGrammar(base.GRAMMAR_FULL) + \
-                pp.Optional(pp.pythonStyleComment)  + pp.StringEnd()
+                pp.Optional(pp.pythonStyleComment) + pp.StringEnd()
         elif grammar_type == base.GRAMMAR_CHKCMD:
             self.check_grammar = self.__buildGrammar(base.GRAMMAR_CHKCMD)
     
     def _displayhook(self,item):
         if item is not None:
-            self.__display.open()
             self.__display.writeObject(item)
-            self.__display.close()
 
     def _precompile(self,source):
         """Look for any internal commands in 'source' and replace them with
@@ -166,9 +191,9 @@ class Interpreter(object):
         lines = source.split('\n')
         result = []
         multiline = None
-        terminated = False
         i, n = 0, len(lines)
         try:
+            terminated = False
             while i < n:
                 line = lines[i].rstrip()
                 if multiline is None:
@@ -266,7 +291,8 @@ class Interpreter(object):
         try:
             #print 'Cmd:',source
             source = self._precompile(source)
-            #print 'Exe:',source
+            if self.debug_calls:
+                print 'Calling',source
         except (OverflowError, SyntaxError, ValueError):
             self.showsyntaxerror(filename,line_offset)
             return False
@@ -284,7 +310,7 @@ class Interpreter(object):
             # Case 2
             return True
         # Case 3
-        print 'Exe:',source.replace('\n','\n...: ')
+        #print 'Exe:',source.replace('\n','\n...: ')
         self.runcode(code)
         return False
 
@@ -380,21 +406,20 @@ class Interpreter(object):
             tblist = tb = None
         map(self.writeErr, list)
 
-    def writeErr(self, data):
+    def writeErr(self, data=''):
         """Write a string to error output.
 
-        It writes to LineDisplay (context main.stderr) provided by ui_provider.
+        It writes to UI_TEXT Display (context main.stderr) provided by ui_provider.
         """
-
         self.__errout.write(data)
 
-    def write(self, data):
+    def write(self, data=''):
         """Write a string to standard output.
 
-        It writes to LineDisplay (context main) provided by ui_provider.
+        It writes to UI_TEXT Display (context main) provided by ui_provider.
         """
 
-        self.__errout.write(data)
+        self.__out.write(data)
 
     def isCmd(self, cmdstr):
         """Returns True if 'cmdstr' is internal command. However, it doesn't 
@@ -417,7 +442,8 @@ class Console(Interpreter):
     using the familiar sys.ps1 and sys.ps2, and input buffering.
     """
 
-    def __init__(self, locals=None, filename="<console>", ui_provider=None):
+    def __init__(self, packages=[], locals=None, filename="<console>", 
+                  ui_provider=None, ui_factory=None):
         """Constructor.
 
         The optional locals argument will be passed to the
@@ -430,7 +456,7 @@ class Console(Interpreter):
         it defaults to base.UserInterfaceProvider.
         """
 
-        super(Console,self).__init__(locals,ui_provider)
+        super(Console,self).__init__(packages,locals,ui_provider,ui_factory)
         self.filename = filename
         self.resetbuffer()
 
@@ -473,7 +499,7 @@ class Console(Interpreter):
                 else:
                     prompt = sys.ps1
                 try:
-                    line = self.raw_input(prompt)
+                    line = self.ui_provider.raw_input(prompt)
                 except EOFError:
                     self.write("\n")
                     break
@@ -505,19 +531,6 @@ class Console(Interpreter):
             self.resetbuffer()
         return more
 
-    def raw_input(self, prompt=""):
-        """Write a prompt and read a line.
-
-        The returned line does not include the trailing newline.
-        When the user enters the EOF key sequence, EOFError is raised.
-
-        The base implementation uses the built-in function
-        raw_input(); a subclass may replace this with a different
-        implementation.
-        """
-
-        return raw_input(prompt)
-
 
 def interact(banner=None, readfunc=None, local=None):
     """Closely emulate the interactive Python interpreter.
@@ -528,9 +541,9 @@ def interact(banner=None, readfunc=None, local=None):
 
     Arguments (all optional, all default to None):
 
-    banner -- passed to InteractiveConsole.interact()
-    readfunc -- if not None, replaces InteractiveConsole.raw_input()
-    local -- passed to InteractiveInterpreter.__init__()
+    :banner: passed to InteractiveConsole.interact()
+    :readfunc: if not None, replaces InteractiveConsole.raw_input()
+    :local: passed to InteractiveInterpreter.__init__()
     """
 
     console = Console(local)
@@ -543,6 +556,3 @@ def interact(banner=None, readfunc=None, local=None):
             pass
     console.interact(banner)
 
-if __name__ == '__main__':
-    interact()
-
